@@ -28,9 +28,9 @@ def concordance_index(event_times, predicted_event_times, event_observed=None):
     reducing errors. Statistics in Medicine 1996;15(4):361-87.
 
     Parameters:
-      event_times: a (nx1) array of observed survival times.
-      predicted_event_times: a (nx1) array of predicted survival times.
-      event_observed: a (nx1) array of censorship flags, 1 if observed,
+      event_times: a (n,) array of observed survival times.
+      predicted_event_times: a (n,) array of predicted survival times.
+      event_observed: a (n,) array of censorship flags, 1 if observed,
                       0 if not. Default assumes all observed.
 
     Returns:
@@ -43,9 +43,9 @@ def concordance_index(event_times, predicted_event_times, event_observed=None):
         event_observed = np.ones(event_times.shape[0], dtype=float)
 
     if event_times.shape != predicted_event_times.shape:
-        raise ValueError("Event times and predictions must have the same shape!")
+        raise ValueError("Event times and predictions must have the same shape")
     if event_times.ndim != 1:
-        raise ValueError("Event times can only be 1-dimensional!")
+        raise ValueError("Event times can only be 1-dimensional: (n,)")
 
     # 100 times faster to calculate in Fortran
     return _cindex(event_times,
@@ -53,7 +53,8 @@ def concordance_index(event_times, predicted_event_times, event_observed=None):
                    event_observed)
 
 
-def logrank_test(event_times_A, event_times_B, event_observed_A=None, event_observed_B=None, alpha=0.95, t_0=-1, **kwargs):
+def logrank_test(event_times_A, event_times_B, event_observed_A=None, event_observed_B=None, 
+                  alpha=0.95, t_0=-1, suppress_print=False, **kwargs):
     """
     Measures and reports on whether two intensity processes are different. That is, given two
     event series, determines whether the data generating processes are statistically different.
@@ -72,6 +73,7 @@ def logrank_test(event_times_A, event_times_B, event_observed_A=None, event_obse
       censorship_bar: a (nx1) array of censorship flags, 1 if observed, 0 if not. Default assumes all observed.
       t_0: the period under observation, -1 for all time.
       alpha: the level of signifiance
+      suppress_print: if True, do not print the summary. Default False.
       kwargs: add keywords and meta-data to the experiment summary
 
     Returns
@@ -89,10 +91,12 @@ def logrank_test(event_times_A, event_times_B, event_observed_A=None, event_obse
     event_times = np.r_[event_times_A, event_times_B]
     groups = np.r_[np.zeros(event_times_A.shape[0]), np.ones(event_times_B.shape[0])]
     event_observed = np.r_[event_observed_A, event_observed_B]
-    return multivariate_logrank_test(event_times, groups, event_observed, alpha=alpha, t_0=t_0, **kwargs)
+    return multivariate_logrank_test(event_times, groups, event_observed, 
+                                          alpha=alpha, t_0=t_0, suppress_print=suppress_print, **kwargs)
 
 
-def pairwise_logrank_test(event_durations, groups, event_observed=None, alpha=0.95, t_0=-1, bonferroni=True, **kwargs):
+def pairwise_logrank_test(event_durations, groups, event_observed=None,
+                          alpha=0.95, t_0=-1, bonferroni=True, suppress_print=False, **kwargs):
     """
     Perform the logrank test pairwise for all n>2 unique groups (use the more appropriate logrank_test for n=2).
     We have to be careful here: if there are n groups, then there are n*(n-1)/2 pairs -- so many pairs increase
@@ -109,6 +113,7 @@ def pairwise_logrank_test(event_durations, groups, event_observed=None, alpha=0.
       t_0: the final time to compare the series' up to. Defaults to all.
       bonferroni: If true, uses the Bonferroni correction to compare the M=n(n-1)/2 pairs, i.e alpha = alpha/M
             See (here)[http://en.wikipedia.org/wiki/Bonferroni_correction].
+      suppress_print: if True, do not print the summary. Default False.
       kwargs: add keywords and meta-data to the experiment summary.
 
     Returns:
@@ -134,11 +139,11 @@ def pairwise_logrank_test(event_durations, groups, event_observed=None, alpha=0.
     if event_observed is None:
         event_observed = np.ones((event_durations.shape[0], 1))
 
-    # if they pass in a dataframe
-    try:
-        unique_groups = np.unique(groups).values
-    except:
-        unique_groups = np.unique(groups)
+    n = max(event_durations.shape)
+    assert n == max(event_durations.shape) == max(event_observed.shape), "inputs must be of the same length."
+    groups, event_durations, event_observed = map(lambda x: pd.Series(np.reshape(x, (n,))), [groups, event_durations, event_observed])
+
+    unique_groups = np.unique(groups)
 
     n = unique_groups.shape[0]
 
@@ -158,10 +163,11 @@ def pairwise_logrank_test(event_durations, groups, event_observed=None, alpha=0.
         g1, g2 = unique_groups[[i1, i2]]
         ix1, ix2 = (groups == g1), (groups == g2)
         test_name = str(g1) + " vs. " + str(g2)
-        summary, p_value, result = logrank_test(event_durations[ix1], event_durations[ix2],
-                                                event_observed[ix1], event_observed[ix2],
+        summary, p_value, result = logrank_test(event_durations.ix[ix1], event_durations.ix[ix2],
+                                                event_observed.ix[ix1], event_observed.ix[ix2],
                                                 alpha=alpha, t_0=t_0, use_bonferroni=bonferroni,
-                                                test_name=test_name, **kwargs)
+                                                test_name=test_name, suppress_print=suppress_print,
+                                                **kwargs)
         T[i1, i2], T[i2, i1] = result, result
         P[i1, i2], P[i2, i1] = p_value, p_value
         S[i1, i2], S[i2, i1] = summary, summary
@@ -169,7 +175,8 @@ def pairwise_logrank_test(event_durations, groups, event_observed=None, alpha=0.
     return [pd.DataFrame(x, columns=unique_groups, index=unique_groups) for x in [S, P, T]]
 
 
-def multivariate_logrank_test(event_durations, groups, event_observed=None, alpha=0.95, t_0=-1, **kwargs):
+def multivariate_logrank_test(event_durations, groups, event_observed=None, 
+                              alpha=0.95, t_0=-1, suppress_print=False, **kwargs):
     """
     This test is a generalization of the logrank_test: it can deal with n>2 populations (and should
       be equal when n=2):
@@ -184,6 +191,7 @@ def multivariate_logrank_test(event_durations, groups, event_observed=None, alph
           to all observed.
       alpha: the level of signifiance desired.
       t_0: the final time to compare the series' up to. Defaults to all.
+      suppress_print: if True, do not print the summary. Default False.
       kwargs: add keywords and meta-data to the experiment summary.
 
     Returns:
@@ -192,11 +200,13 @@ def multivariate_logrank_test(event_durations, groups, event_observed=None, alph
       test_result: True if reject the null, (pendantically) None if we can't reject the null.
 
     """
-    assert event_durations.shape[0] == groups.shape[0], "event_durations must be the same shape as groups"
-
     if event_observed is None:
         event_observed = np.ones((event_durations.shape[0], 1))
 
+    n = max(event_durations.shape)
+    assert n == max(event_durations.shape) == max(event_observed.shape), "inputs must be of the same length."
+    groups, event_durations, event_observed = map(lambda x: pd.Series(np.reshape(x, (n,))), [groups, event_durations, event_observed])
+
     unique_groups, rm, obs, _ = group_survival_table_from_events(groups, event_durations, event_observed, np.zeros_like(event_durations), t_0)
     n_groups = unique_groups.shape[0]
 
@@ -219,7 +229,7 @@ def multivariate_logrank_test(event_durations, groups, event_observed=None, alph
     V[ix, ix] = V[ix, ix] + ev
 
     # take the first n-1 groups
-    U = Z_j.ix[:-1].dot(np.linalg.inv(V[:-1, :-1]).dot(Z_j.ix[:-1]))  # Z.T*inv(V)*Z
+    U = Z_j.ix[:-1].dot(np.linalg.pinv(V[:-1, :-1]).dot(Z_j.ix[:-1]))  # Z.T*inv(V)*Z
 
     # compute the p-values and tests
     test_result, p_value = chisq_test(U, n_groups - 1, alpha)
@@ -227,7 +237,8 @@ def multivariate_logrank_test(event_durations, groups, event_observed=None, alph
                                    alpha=alpha, null_distribution='chi squared',
                                    df=n_groups - 1, **kwargs)
 
-    print(summary)
+    if not suppress_print:
+      print(summary)
     return summary, p_value, test_result
 
 
