@@ -1,11 +1,16 @@
+# -*- coding: utf-8 -*-
 """
 python -m lifelines.tests.test_suit
 
 """
 from __future__ import print_function
 
+import os
 import unittest
-from StringIO import StringIO
+try:
+    from StringIO import StringIO
+except ImportError:
+    from io import StringIO
 
 import numpy as np
 import numpy.testing as npt
@@ -14,12 +19,16 @@ import matplotlib.pyplot as plt
 import pandas as pd
 
 from ..estimation import KaplanMeierFitter, NelsonAalenFitter, AalenAdditiveFitter, \
-                         median_survival_times, BreslowFlemingHarringtonFitter, BayesianFitter
-from ..statistics import logrank_test, multivariate_logrank_test, pairwise_logrank_test
+    median_survival_times, BreslowFlemingHarringtonFitter, BayesianFitter, \
+    CoxPHFitter
+
+from ..statistics import (logrank_test, multivariate_logrank_test,
+                          pairwise_logrank_test, concordance_index)
 from ..generate_datasets import *
 from ..plotting import plot_lifetimes
 from ..utils import *
-from ..datasets import lcd_dataset
+from ..datasets import lcd_dataset, rossi_dataset, waltons_dataset, regression_dataset
+
 
 class MiscTests(unittest.TestCase):
 
@@ -61,15 +70,6 @@ class MiscTests(unittest.TestCase):
         npt.assert_almost_equal(C, np.array([1, 0, 0], dtype=bool))
         return
 
-    """
-    def test_datetimes_to_durations_max_dates(self):
-        start_date = ['2013-10-10', '2013-10-09', '2013-10-08']
-        end_date =  ['2014-10-10', '2014-10-09', '2014-10-08']
-        T,C = datetimes_to_durations(start_date, end_date, fill_date = "2014-10-09" )
-        npt.assert_almost_equal(C, np.array([1,0,0], dtype=bool) )
-        return
-    """
-
     def test_survival_table_to_events(self):
         T, C = np.array([1, 2, 3, 4, 4, 5]), np.array([1, 0, 1, 1, 1, 1])
         d = survival_table_from_events(T, C, np.zeros_like(T))
@@ -87,9 +87,13 @@ class MiscTests(unittest.TestCase):
         npt.assert_array_equal(kmf.confidence_interval_.columns, expected)
 
     def test_group_survival_table_from_events_works_with_series(self):
-        df = pd.DataFrame([[1,True,3],[1,True,3],[4,False,2]], columns = ['duration', 'E', 'G'])
-        ug,_,_,_ = group_survival_table_from_events(df.G, df.duration, df.E, np.array([[0,0,0]]))
-        npt.assert_array_equal(ug, np.array([3,2]))
+        df = pd.DataFrame([[1, True, 3], [1, True, 3], [4, False, 2]], columns=['duration', 'E', 'G'])
+        ug, _, _, _ = group_survival_table_from_events(df.G, df.duration, df.E, np.array([[0, 0, 0]]))
+        npt.assert_array_equal(ug, np.array([3, 2]))
+
+    def test_cross_validator(self):
+        cf = CoxPHFitter()
+        k_fold_cross_validation(cf, regression_dataset, duration_col='T', event_col='E', k=3)
 
 
 class StatisticalTests(unittest.TestCase):
@@ -140,14 +144,12 @@ class StatisticalTests(unittest.TestCase):
         data1 = np.random.exponential(5, size=(200, 1))
         data2 = np.random.exponential(5, size=(200, 1))
         summary, p_value, result = logrank_test(data1, data2)
-        print(summary)
         self.assertTrue(result is None)
 
     def test_unequal_intensity(self):
         data1 = np.random.exponential(5, size=(200, 1))
         data2 = np.random.exponential(1, size=(200, 1))
         summary, p_value, result = logrank_test(data1, data2)
-        print(summary)
         self.assertTrue(result)
 
     def test_unequal_intensity_event_observed(self):
@@ -156,19 +158,16 @@ class StatisticalTests(unittest.TestCase):
         eventA = np.random.binomial(1, 0.5, size=(200, 1))
         eventB = np.random.binomial(1, 0.5, size=(200, 1))
         summary, p_value, result = logrank_test(data1, data2, event_observed_A=eventA, event_observed_B=eventB)
-        print(summary)
         self.assertTrue(result)
 
     def test_integer_times_logrank_test(self):
         data1 = np.random.exponential(5, size=(200, 1)).astype(int)
         data2 = np.random.exponential(1, size=(200, 1)).astype(int)
         summary, p_value, result = logrank_test(data1, data2)
-        print(summary)
         self.assertTrue(result)
 
-    def test_waltons_data(self):
+    def test_waltons_dataset(self):
         summary, p_value, result = logrank_test(waltonT1, waltonT2)
-        print(summary)
         self.assertTrue(result)
 
     def test_smoothing_hazard_ties(self):
@@ -209,19 +208,16 @@ class StatisticalTests(unittest.TestCase):
         g = np.random.binomial(2, 0.5, size=300)
         T[g == 1] = np.random.exponential(6, size=(g == 1).sum())
         s, _, result = multivariate_logrank_test(T, g)
-        print(s)
-        self.assertTrue(result == True)
+        self.assertTrue(result)
 
     def test_multivariate_equal_intensities(self):
         T = np.random.exponential(10, size=300)
         g = np.random.binomial(2, 0.5, size=300)
         s, _, result = multivariate_logrank_test(T, g)
-        print(s)
         self.assertTrue(result is None)
 
-    def test_pairwise_waltons_data(self):
-        _, _, R = pairwise_logrank_test(waltonT, waltonG)
-        print(R)
+    def test_pairwise_waltons_dataset(self):
+        _, _, R = pairwise_logrank_test(waltons_dataset['T'], waltons_dataset['group'])
         self.assertTrue(R.values[0, 1])
 
     def test_pairwise_logrank_test(self):
@@ -230,7 +226,21 @@ class StatisticalTests(unittest.TestCase):
         S, P, R = pairwise_logrank_test(T, g, alpha=0.95)
         V = np.array([[np.nan, None, None], [None, np.nan, None], [None, None, np.nan]])
         npt.assert_array_equal(R, V)
-    
+
+    def test_multivariate_inputs(self):
+        T = np.array([1,2,3])
+        E = np.array([1,1,0], dtype=bool)
+        G = np.array([1,2,1])
+        multivariate_logrank_test(T,G,E)
+        pairwise_logrank_test(T,G,E)
+
+        T = pd.Series(T)
+        E = pd.Series(E)
+        G = pd.Series(G)
+        multivariate_logrank_test(T,G,E)
+        pairwise_logrank_test(T,G,E)
+
+
     def test_lists_to_KaplanMeierFitter(self):
         T = [2, 3, 4., 1., 6, 5.]
         C = [1, 0, 0, 0, 1, 1]
@@ -273,6 +283,7 @@ class StatisticalTests(unittest.TestCase):
         T, C = exponential_survival_data(N, 0.2, scale=10)
         self.assertTrue(abs(C.mean() - 0.8) < 0.02)
 
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_exponential_data_sets_fit(self):
         N = 20000
         T, C = exponential_survival_data(N, 0.2, scale=10)
@@ -282,41 +293,43 @@ class StatisticalTests(unittest.TestCase):
 
     def test_subtraction_function(self):
         kmf = KaplanMeierFitter()
-        kmf.fit(waltonT)
+        kmf.fit(waltons_dataset['T'])
         npt.assert_array_almost_equal(kmf.subtract(kmf).sum().values, 0.0)
 
     def test_divide_function(self):
         kmf = KaplanMeierFitter()
-        kmf.fit(waltonT)
+        kmf.fit(waltons_dataset['T'])
         npt.assert_array_almost_equal(np.log(kmf.divide(kmf)).sum().values, 0.0)
 
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_kmf_minimum_observation_bias(self):
         N = 250
         kmf = KaplanMeierFitter()
         T, C = exponential_survival_data(N, 0.2, scale=10)
         B, _ = exponential_survival_data(N, 0.0, scale=2)
-        kmf.fit(T,C, entry=B)
+        kmf.fit(T, C, entry=B)
         kmf.plot()
         plt.title("Should have larger variances in the tails")
 
     def test_stat_error(self):
-        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30, \
-            41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
-        observations = np.array([ 1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31, \
-            22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
+        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30,
+                           41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
+        observations = np.array([1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31,
+                                 22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
         kmf = KaplanMeierFitter()
         with self.assertRaises(StatError):
             kmf.fit(observations, entry=births)
 
     def test_BHF_fit(self):
         bfh = BreslowFlemingHarringtonFitter()
-        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30, \
-           41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
-        observations = np.array([ 1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31, \
-           22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
+        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30,
+                           41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
+        observations = np.array([1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31,
+                                 22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
         bfh.fit(observations, entry=births)
-        return 
+        return
 
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_bayesian_fitter_low_data(self):
         bf = BayesianFitter(samples=15)
         bf.fit(waltonT1)
@@ -326,13 +339,15 @@ class StatisticalTests(unittest.TestCase):
         plt.show()
         return
 
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_bayesian_fitter_large_data(self):
         bf = BayesianFitter()
-        bf.fit(np.random.exponential(10,size=1000))
+        bf.fit(np.random.exponential(10, size=1000))
         bf.plot()
         plt.show()
         return
 
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_kmf_left_censorship_plots(self):
         kmf = KaplanMeierFitter()
         kmf.fit(lcd_dataset['alluvial_fan']['T'], lcd_dataset['alluvial_fan']['C'], left_censorship=True, label='alluvial_fan')
@@ -345,15 +360,13 @@ class StatisticalTests(unittest.TestCase):
 
     def test_kmf_left_censorship_stats(self):
         T = [3, 5, 5, 5, 6, 6, 10, 12]
-        C = [1,0,0,1,1,1,0,1]
+        C = [1, 0, 0, 1, 1, 1, 0, 1]
         kmf = KaplanMeierFitter()
-        kmf.fit(T,C, left_censorship=True)
-
-
+        kmf.fit(T, C, left_censorship=True)
 
     def kaplan_meier(self, censor=False):
-        km = np.zeros((len(self.lifetimes.keys()), 1))
-        ordered_lifetimes = np.sort(self.lifetimes.keys())
+        km = np.zeros((len(list(self.lifetimes.keys())), 1))
+        ordered_lifetimes = np.sort(list(self.lifetimes.keys()))
         N = len(LIFETIMES)
         v = 1.
         n = N * 1.0
@@ -373,8 +386,8 @@ class StatisticalTests(unittest.TestCase):
         return km.reshape(len(km), 1)
 
     def nelson_aalen(self, censor=False):
-        na = np.zeros((len(self.lifetimes.keys()), 1))
-        ordered_lifetimes = np.sort(self.lifetimes.keys())
+        na = np.zeros((len(list(self.lifetimes.keys())), 1))
+        ordered_lifetimes = np.sort(list(self.lifetimes.keys()))
         N = len(LIFETIMES)
         v = 0.
         n = N * 1.0
@@ -393,6 +406,25 @@ class StatisticalTests(unittest.TestCase):
             na = np.insert(na, 0, 0.)
         return na.reshape(len(na), 1)
 
+    def test_concordance_index(self):
+        size = 1000
+        T = np.random.normal(size=size)
+        P = np.random.normal(size=size)
+        C = np.random.choice([0, 1], size=size)
+        Z = np.zeros_like(T)
+
+        # Zeros is exactly random
+        self.assertTrue(concordance_index(T, Z) == 0.5)
+        self.assertTrue(concordance_index(T, Z, C) == 0.5)
+
+        # Itself is 1
+        self.assertTrue(concordance_index(T, T) == 1.0)
+        self.assertTrue(concordance_index(T, T, C) == 1.0)
+
+        # Random is close to 0.5
+        self.assertTrue(abs(concordance_index(T, P) - 0.5) < 0.05)
+        self.assertTrue(abs(concordance_index(T, P, C) - 0.5) < 0.05)
+
 
 class AalenAdditiveModelTests(unittest.TestCase):
 
@@ -402,7 +434,7 @@ class AalenAdditiveModelTests(unittest.TestCase):
     def test_large_dimensions_for_recursion_error(self):
         n = 500
         d = 50
-        X = pd.DataFrame(np.random.randn(n,d))
+        X = pd.DataFrame(np.random.randn(n, d))
         T = np.random.exponential(size=n)
         X['T'] = T
         X['E'] = 1
@@ -414,7 +446,7 @@ class AalenAdditiveModelTests(unittest.TestCase):
     def test_tall_data_points(self):
         n = 20000
         d = 2
-        X = pd.DataFrame(np.random.randn(n,d))
+        X = pd.DataFrame(np.random.randn(n, d))
         T = np.random.exponential(size=n)
         X['T'] = T
         X['E'] = 1
@@ -422,13 +454,13 @@ class AalenAdditiveModelTests(unittest.TestCase):
         aaf.fit(X)
 
         return True
-    
+
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_aaf_panel_dataset(self):
         aaf = AalenAdditiveFitter()
-        aaf.fit(panel_dataset, id_col='id',duration_col='t', event_col='E')
+        aaf.fit(panel_dataset, id_col='id', duration_col='t', event_col='E')
         aaf.plot()
         return
-    
 
     def test_aalen_additive_median_predictions_split_data(self):
         # This tests to make sure that my median predictions statisfy
@@ -446,10 +478,10 @@ class AalenAdditiveModelTests(unittest.TestCase):
         aaf.fit(X)
 
         # predictions
-        T_pred = aaf.predict_median(X[range(6)])
+        T_pred = aaf.predict_median(X[list(range(6))])
         self.assertTrue(abs((T_pred.values > T).mean() - 0.5) < 0.05)
 
-
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_aalen_additive_fit_no_censor(self):
         # this is a visual test of the fitting the cumulative
         # hazards.
@@ -458,22 +490,23 @@ class AalenAdditiveModelTests(unittest.TestCase):
         timeline = np.linspace(0, 70, 10000)
         hz, coef, X = generate_hazard_rates(n, d, timeline)
         X.columns = coef.columns
-        cumulative_hazards = pd.DataFrame(cumulative_integral(coef.values, timeline), 
+        cumulative_hazards = pd.DataFrame(cumulative_integral(coef.values, timeline),
                                           index=timeline, columns=coef.columns)
         T = generate_random_lifetimes(hz, timeline)
         X['T'] = T
-        X['E'] = np.random.binomial(1,1,n)
+        X['E'] = np.random.binomial(1, 1, n)
         aaf = AalenAdditiveFitter(penalizer=1., fit_intercept=False)
         aaf.fit(X)
 
-        for i in range(d+1):
-            ax = plt.subplot(d+1,1,i+1)
+        for i in range(d + 1):
+            ax = plt.subplot(d + 1, 1, i + 1)
             col = cumulative_hazards.columns[i]
-            ax = cumulative_hazards[col].ix[:15].plot(legend=False,ax=ax)
-            ax = aaf.plot(ix=slice(0,15),ax=ax, columns=[col], legend=False)
+            ax = cumulative_hazards[col].ix[:15].plot(legend=False, ax=ax)
+            ax = aaf.plot(ix=slice(0, 15), ax=ax, columns=[col], legend=False)
         plt.show()
         return
 
+    @unittest.skipUnless("DISPLAY" in os.environ, "requires display")
     def test_aalen_additive_fit_with_censor(self):
         # this is a visual test of the fitting the cumulative
         # hazards.
@@ -482,32 +515,33 @@ class AalenAdditiveModelTests(unittest.TestCase):
         timeline = np.linspace(0, 70, 10000)
         hz, coef, X = generate_hazard_rates(n, d, timeline)
         X.columns = coef.columns
-        cumulative_hazards = pd.DataFrame(cumulative_integral(coef.values, timeline), 
+        cumulative_hazards = pd.DataFrame(cumulative_integral(coef.values, timeline),
                                           index=timeline, columns=coef.columns)
         T = generate_random_lifetimes(hz, timeline)
         X['T'] = T
-        X['E'] = np.random.binomial(1,0.99,n)
+        X['E'] = np.random.binomial(1, 0.99, n)
         aaf = AalenAdditiveFitter(penalizer=1., fit_intercept=False)
         aaf.fit(X)
 
-        for i in range(d+1):
-            ax = plt.subplot(d+1,1,i+1)
+        for i in range(d + 1):
+            ax = plt.subplot(d + 1, 1, i + 1)
             col = cumulative_hazards.columns[i]
-            ax = cumulative_hazards[col].ix[:15].plot(legend=False,ax=ax)
-            ax = aaf.plot(ix=slice(0,15),ax=ax, columns=[col], legend=False)
+            ax = cumulative_hazards[col].ix[:15].plot(legend=False, ax=ax)
+            ax = aaf.plot(ix=slice(0, 15), ax=ax, columns=[col], legend=False)
         plt.show()
         return
 
     def test_dataframe_input_with_nonstandard_index(self):
 
-        df = pd.DataFrame([(16,True,True), (1,True,True), (4,False,True)], 
+        df = pd.DataFrame([(16, True, True), (1, True, True), (4, False, True)],
                           columns=['duration', 'done_feeding', 'white'],
-                          index=['a','b','c'])
+                          index=['a', 'b', 'c'])
         aaf = AalenAdditiveFitter()
         aaf.fit(df, duration_col='duration', event_col='done_feeding')
-        return 
+        return
 
 
+@unittest.skipUnless("DISPLAY" in os.environ, "requires display")
 class PlottingTests(unittest.TestCase):
 
     def test_aalen_additive_plot(self):
@@ -583,12 +617,12 @@ class PlottingTests(unittest.TestCase):
         return True
 
     def test_ix_slicing(self):
-        naf = NelsonAalenFitter().fit(waltonT)
+        naf = NelsonAalenFitter().fit(waltons_dataset['T'])
         self.assertTrue(naf.cumulative_hazard_.ix[0:10].shape[0] == 4)
         return
 
     def test_iloc_slicing(self):
-        naf = NelsonAalenFitter().fit(waltonT)
+        naf = NelsonAalenFitter().fit(waltons_dataset['T'])
         self.assertTrue(naf.cumulative_hazard_.iloc[0:10].shape[0] == 10)
         self.assertTrue(naf.cumulative_hazard_.iloc[0:-1].shape[0] == 32)
         return
@@ -668,76 +702,82 @@ class PlottingTests(unittest.TestCase):
         C2 = np.random.binomial(1, 0.95, size=200)
         kmf = KaplanMeierFitter()
         kmf.fit(data1, C1, label='test label 1')
-        ax = kmf.plot(flat=True, censor_styles={'marker':'+', 'mew':2, 'ms':7})
+        ax = kmf.plot(flat=True, censor_styles={'marker': '+', 'mew': 2, 'ms': 7})
         kmf.fit(data2, C2, label='test label 2')
-        kmf.plot(ax=ax, censor_styles={'marker':'o', 'ms':7}, flat=True)
+        kmf.plot(ax=ax, censor_styles={'marker': 'o', 'ms': 7}, flat=True)
         plt.title("testing kmf flat styling + marker")
         return
 
     def test_flat_style_no_censor(self):
         data1 = np.random.exponential(10, size=200)
         kmf = KaplanMeierFitter()
-        kmf.fit(data1,label='test label 1')
-        ax = kmf.plot(flat=True, censor_styles={'marker':'+', 'mew':2, 'ms':7})
+        kmf.fit(data1, label='test label 1')
+        ax = kmf.plot(flat=True, censor_styles={'marker': '+', 'mew': 2, 'ms': 7})
         return
 
 
+class CoxRegressionTests(unittest.TestCase):
+
+    def test_efron_computed_by_hand_examples(self):
+        score_efron = CoxPHFitter()._score_efron
+        hessian_efron = CoxPHFitter()._hessian_efron
+
+        X = data_nus['x'][:, None]
+        T = data_nus['t']
+        E = data_nus['E']
+
+        # tests from http://courses.nus.edu.sg/course/stacar/internet/st3242/handouts/notes3.pdf
+        beta = np.array([[0]])
+
+        l = -hessian_efron(X, beta, T, E)
+        u = score_efron(X, beta, T, E)
+        assert np.abs(l[0][0] - 77.13) < 0.05
+        assert np.abs(u[0] - -2.51) < 0.05
+        beta = beta + u / l
+        assert np.abs(beta - -0.0326) < 0.05
+
+        l = -hessian_efron(X, beta, T, E)
+        u = score_efron(X, beta, T, E)
+        assert np.abs(l[0][0] - 72.83) < 0.05
+        assert np.abs(u[0] - -0.069) < 0.05
+        beta = beta + u / l
+        assert np.abs(beta - -0.0325) < 0.01
+
+        l = -hessian_efron(X, beta, T, E)
+        u = score_efron(X, beta, T, E)
+        assert np.abs(l[0][0] - 72.70) < 0.01
+        assert np.abs(u[0] - -0.000061) < 0.01
+        beta = beta + u / l
+        assert np.abs(beta - -0.0335) < 0.01
+
+    def test_efron_newtons_method(self):
+        newton = CoxPHFitter()._newton_rhapdson
+        X, T, E = data_nus['x'][:, None], data_nus['t'], data_nus['E']
+        assert np.abs(newton(X, T, E)[0][0] - -0.0335) < 0.0001
+
+    def test_fit_method(self):
+        cf = CoxPHFitter()
+        cf.fit(data_nus, duration_col='t', event_col='E')
+        self.assertTrue(np.abs(cf.hazards_.ix[0][0] - -0.0335) < 0.0001)
+
+    def test_output_against_R(self):
+        # from http://cran.r-project.org/doc/contrib/Fox-Companion/appendix-cox-regression.pdf
+        expected = np.array([[-0.3794, -0.0574, 0.3139, -0.1498, -0.4337, -0.0849,  0.0915]])
+        df = rossi_dataset
+        cf = CoxPHFitter()
+        cf.fit(df, duration_col='week', event_col='arrest')
+        npt.assert_array_almost_equal(cf.hazards_.values, expected, decimal=3)
+
+
 # some data
 LIFETIMES = np.array([2, 4, 4, 4, 5, 7, 10, 11, 11, 12])
 OBSERVED = np.array([1, 1, 0, 1, 0, 1, 1, 1, 1, 0])
 N = len(LIFETIMES)
 
-waltonT1 = np.array([6.,13.,13.,13.,19.,19.,19.,26.,26.,26.,26.,26.,33.,33.,47.,62.,62.,9.,9.,9.,15.,15.,22.,22.,22.,22.,29.,29.,29.,29.,29.,36.,36.,43.])
-waltonT2 = np.array([33.,54.,54.,61.,61.,61.,61.,61.,61.,61.,61.,61.,61.,61.,69.,69.,69.,69.,69.,69.,69.,69.,69.,69.,69.,32.,53.,53.,60.,60.,60.,60.,60.,
-                        68.,68.,68.,68.,68.,68.,68.,68.,68.,68.,75.,17.,51.,51.,51.,58.,58.,58.,58.,66.,66.,7.,7.,41.,41.,41.,41.,41.,41.,41.,48.,48.,48.,
-                        48.,48.,48.,48.,48.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,56.,63.,63.,63.,63.,63.,63.,63.,63.,63.,69.,
-                        69.,38.,38.,45.,45.,45.,45.,45.,45.,45.,45.,45.,45.,53.,53.,53.,53.,53.,60.,60.,60.,60.,60.,60.,60.,60.,60.,60.,60.,66.])
-
-waltonG = np.array(['miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137',
-                    'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137',
-                    'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137',
-                    'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137',
-                    'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137', 'miR-137',
-                    'miR-137', 'miR-137', 'miR-137', 'miR-137', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control', 'control', 'control', 'control', 'control', 'control',
-                    'control'], dtype=object)
-
-waltonT = np.array([6., 13., 13., 13., 19., 19., 19., 26., 26., 26., 26.,
-                    26., 33., 33., 47., 62., 62., 9., 9., 9., 15., 15.,
-                    22., 22., 22., 22., 29., 29., 29., 29., 29., 36., 36.,
-                    43., 33., 54., 54., 61., 61., 61., 61., 61., 61., 61.,
-                    61., 61., 61., 61., 69., 69., 69., 69., 69., 69., 69.,
-                    69., 69., 69., 69., 32., 53., 53., 60., 60., 60., 60.,
-                    60., 68., 68., 68., 68., 68., 68., 68., 68., 68., 68.,
-                    75., 17., 51., 51., 51., 58., 58., 58., 58., 66., 66.,
-                    7., 7., 41., 41., 41., 41., 41., 41., 41., 48., 48.,
-                    48., 48., 48., 48., 48., 48., 56., 56., 56., 56., 56.,
-                    56., 56., 56., 56., 56., 56., 56., 56., 56., 56., 56.,
-                    56., 56., 63., 63., 63., 63., 63., 63., 63., 63., 63.,
-                    69., 69., 38., 38., 45., 45., 45., 45., 45., 45., 45.,
-                    45., 45., 45., 53., 53., 53., 53., 53., 60., 60., 60.,
-                    60., 60., 60., 60., 60., 60., 60., 60., 66.])
-
+#walton's data
+ix = waltons_dataset['group'] == 'miR-137'
+waltonT1 = waltons_dataset.ix[ix]['T']
+waltonT2 = waltons_dataset.ix[~ix]['T']
 
 
 panel_dataset = pd.read_csv(
@@ -773,6 +813,19 @@ panel_dataset = pd.read_csv(
 """))
 
 
+data_nus = pd.DataFrame([
+    [6, 31.4],
+    [98, 21.5],
+    [189, 27.1],
+    [374, 22.7],
+    [1002, 35.7],
+    [1205, 30.7],
+    [2065, 26.5],
+    [2201, 28.3],
+    [2421, 27.9]],
+    columns=['t', 'x'])
+data_nus['E'] = True
+
 
 if __name__ == '__main__':
     unittest.main()
