@@ -1,13 +1,17 @@
-from __future__ import print_function
+# -*- coding: utf-8 -*-
+from __future__ import print_function, division
 
 from datetime import datetime
 
 import numpy as np
+from numpy.random import permutation
 import pandas as pd
 from pandas import to_datetime
 
+
 def coalesce(*args):
-  return next( s for s in args if s is not None)
+    return next(s for s in args if s is not None)
+
 
 def group_survival_table_from_events(groups, durations, event_observed, min_observations, limit=-1):
     """
@@ -18,9 +22,9 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
         groups: a (n,) array of individuals' group ids.
         durations: a (n,)  array of durations of each individual
         event_observed: a (n,) array of event observations, 1 if observed, 0 else.
-        event_observed: a (n,) array of times individual entered study. This is most applicable in 
-                    cases where there is left-truncation, i.e. a individual might enter the 
-                    study late. If not the case, normally set to all zeros. 
+        event_observed: a (n,) array of times individual entered study. This is most applicable in
+                    cases where there is left-truncation, i.e. a individual might enter the
+                    study late. If not the case, normally set to all zeros.
 
     Output:
         - np.array of unique groups
@@ -63,7 +67,7 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
     """
     n = max(groups.shape)
     assert n == max(durations.shape) == max(event_observed.shape) == max(min_observations.shape), "inputs must be of the same length."
-    groups, durations, event_observed, min_observations = map(lambda x: pd.Series(np.reshape(x, (n,))),[groups, durations, event_observed, min_observations])
+    groups, durations, event_observed, min_observations = map(lambda x: pd.Series(np.reshape(x, (n,))), [groups, durations, event_observed, min_observations])
     unique_groups = groups.unique()
 
     # set first group
@@ -74,17 +78,17 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
     B = min_observations[ix]
 
     g_name = str(g)
-    data = survival_table_from_events(T, C, B, 
-                columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name])
+    data = survival_table_from_events(T, C, B,
+                                      columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name])
     for g in unique_groups[1:]:
         ix = groups == g
         T = durations[ix]
         C = event_observed[ix]
         B = min_observations[ix]
         g_name = str(g)
-        data = data.join(survival_table_from_events(T, C, B, 
-                    columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name]),
-                    how='outer')
+        data = data.join(survival_table_from_events(T, C, B,
+                                                    columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name]),
+                                                    how='outer')
     data = data.fillna(0)
     # hmmm pandas its too bad I can't do data.ix[:limit] and leave out the if.
     if int(limit) != -1:
@@ -93,12 +97,12 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
 
 
 def survival_table_from_events(durations, event_observed, min_observations,
-                              columns=["removed", "observed", "censored", 'entrance'], weights=None):
+                               columns=["removed", "observed", "censored", 'entrance'], weights=None):
     """
     Parameters:
         durations: (n,1) array of event times (durations individual was observed for)
         event_observed: (n,1) boolean array, 1 if observed event, 0 is censored event.
-        min_observations: used for left truncation data. Sometimes subjects will show 
+        min_observations: used for left truncation data. Sometimes subjects will show
           up late in the study. min_observations is a (n,1) array of positive numbers representing
           when the subject was first observed. A subject's life is then [min observation + duration observed]
         columns: a 3-length array to call the, in order, removed individuals, observed deaths
@@ -127,7 +131,7 @@ def survival_table_from_events(durations, event_observed, min_observations,
         15              2         2         0
 
     """
-    #deal with deaths and censorships
+    # deal with deaths and censorships
 
     durations = np.asarray(durations) + min_observations
     df = pd.DataFrame(durations, columns=["event_at"])
@@ -136,13 +140,13 @@ def survival_table_from_events(durations, event_observed, min_observations,
     death_table = df.groupby("event_at").sum()
     death_table[columns[2]] = (death_table[columns[0]] - death_table[columns[1]]).astype(int)
 
-    #deal with late births
-    births = pd.DataFrame( min_observations, columns=['event_at'])
+    # deal with late births
+    births = pd.DataFrame(min_observations, columns=['event_at'])
     births[columns[3]] = 1
     births_table = births.groupby('event_at').sum()
- 
-    #this next line can be optimized for when min_observerations is all zeros.     
-    event_table = death_table.join(births_table, how='outer', sort=True).fillna(0) #http://wesmckinney.com/blog/?p=414
+
+    # this next line can be optimized for when min_observerations is all zeros.
+    event_table = death_table.join(births_table, how='outer', sort=True).fillna(0)  # http://wesmckinney.com/blog/?p=414
 
     return event_table
 
@@ -229,10 +233,12 @@ def datetimes_to_durations(start_times, end_times, fill_date=datetime.today(), f
 
 
 class StatError(Exception):
-     def __init__(self, msg):
-         self.msg = msg
-     def __str__(self):
-         return repr(self.msg)
+
+    def __init__(self, msg):
+        self.msg = msg
+
+    def __str__(self):
+        return repr(self.msg)
 
 
 def inv_normal_cdf(p):
@@ -258,31 +264,52 @@ def AandS_approximation(p):
     return t - (c_0 + c_1 * t + c_2 * t ** 2) / (1 + d_1 * t + d_2 * t * t + d_3 * t ** 3)
 
 
-def median_loss(T, T_pred):
-    return np.abs(T - T_pred).mean()
-
-
-def cross_validation(fitter, T, X, event_observed=None, k=5, loss_function="median"):
+def k_fold_cross_validation(fitter, df, duration_col='T', event_col='E', k=5, loss_function="concordance"):
     """
-    This implements the censor-sensative loss function as given in
-    'Prediction Performance of Survival Models', Yan Yuan, 2008
+    Perform cross validation on a dataset. 
+
+    fitter: either an instance of AalenAdditiveFitter or CoxPHFitter.
+    df: a Pandas dataframe with necessary columns `duration_col` and `event_col`, plus
+        other covariates. `duration_col` refers to the lifetimes of the subjects. `event_col`
+        refers to whether the 'death' events was observed: 1 if observed, 0 else (censored).
+    duration_col: the column in dataframe that contains the subjects lifetimes.
+    event_col: the column in dataframe that contains the subject's death observation.    loss functions:
+    k: the number of folds to perform. n/k data will be withheld for testing on.
+    loss_function: "concordance" only.
+            "concordance":  concordance index (C-index) between two series of event times
+
+    Returns:
+        (k,1) array of scores for each fold. 
     """
-    pass
+    from .statistics import concordance_index
 
+    n,d = df.shape
+    scores = np.zeros((k,))
+    df = df.copy()
+    df = df.reindex(np.random.permutation(df.index)).sort(event_col)    
 
-def yaun_loss(fitter, T_true, T_pred, X_train, T_train, event_observed=None):
-    "[WIP]"
-    if event_observed is None:
-        t = T_pred.shape[0]
-        sC = np.ones((t, 1))
-        event_observed = np.ones((t, 1))
-    else:
-        # We are estimating S_c(*|z),is the probability that the
-        # jth individual survived to time yj without being censored
-        fitter.fit(T_train, X_train, event_observed=1 - event_observed, timeline=T_true)
-        sC = fitter.predict_survival_function(X_train).ix[T_true[:, 0]].values.diagonal()[:, None]  # dirty way to get this
+    assignments = np.array((n//k + 1)*list(range(1,k+1)))
+    assignments = assignments[:n]
 
-    return (event_observed * (T_pred - T_true) ** 2 / sC).mean()
+    testing_columns = df.columns - [duration_col, event_col]
+
+    for i in range(1,k+1):
+
+        ix = assignments == i 
+        training_data = df.ix[~ix]
+        testing_data = df.ix[ix]
+
+        T_actual = testing_data[duration_col].values
+        E_actual = testing_data[event_col].values
+        X_testing = testing_data[testing_columns]
+
+        #fit the fitter to the training data
+        fitter.fit(training_data, duration_col=duration_col, event_col=event_col)
+        T_pred = fitter.predict_median(X_testing).values
+
+        scores[i-1] = concordance_index(T_actual, T_pred, E_actual)
+
+    return scores
 
 def epanechnikov_kernel(t, T, bandwidth=1.):
     M = 0.75 * (1 - (t - T) / bandwidth) ** 2
